#!/usr/bin/env python
# coding=utf-8
import sys
import threading
import time

from pymachinetalk.dns_sd import ServiceDiscovery
from pymachinetalk import application


class Logger(object):
    def __init__(self, debug=False):
        self._sd = ServiceDiscovery()

        log = application.ApplicationLog(debug=debug)
        log.log_level = application.log.RTAPI_MSG_DBG
        log.on_message_received.append(self._on_log_message_received)

        self._sd.register(log)

    @staticmethod
    def _on_log_message_received(msg):
        print(msg)

    def start(self):
        self._sd.start()

    def stop(self):
        self._sd.stop()


def main():
    logger = Logger()

    print('starting')
    logger.start()

    try:
        while True:
            time.sleep(0.5)
    except KeyboardInterrupt:
        pass

    print('stopping threads')
    logger.stop()

    # wait for all threads to terminate
    while threading.active_count() > 1:
        time.sleep(0.1)

    print('threads stopped')
    sys.exit(0)


if __name__ == '__main__':
    main()



# coding=utf-8
import ftplib
import os
import threading
from six.moves.urllib.parse import urlparse

from ..common import ComponentBase
from ..dns_sd import ServiceContainer, Service


class ApplicationFile(ComponentBase, ServiceContainer):
    def __init__(self, debug=True):
        self._error_string = ''
        self.on_error_string_changed = []
        ComponentBase.__init__(self)
        ServiceContainer.__init__(self)
        self.debug = debug
        self.state_condition = threading.Condition(threading.Lock())
        self.file_list_lock = threading.Lock()

        self.file_uri = ''
        self.local_file_path = ''
        self.remote_file_path = ''
        self.local_path = ''
        self.remote_path = ''
        self.transfer_state = 'NoTransfer'
        self.bytes_sent = 0.0
        self.bytes_total = 0.0
        self.progress = 0.0
        self.file = None

        self._file_list = []

        self._file_service = Service(type_='file')
        self.add_service(self._file_service)
        self.on_services_ready_changed.append(self._on_services_ready_changed)

    def _on_services_ready_changed(self, ready):
        self.file_uri = self._file_service.uri
        self.ready = ready

    @property
    def error_string(self):
        return self._error_string

    @error_string.setter
    def error_string(self, string):
        if self._error_string is string:
            return
        self._error_string = string
        for cb in self.on_error_string_changed:
            cb(string)

    @property
    def file_list(self):
        with self.file_list_lock:
            return self._file_list

    def _upload_worker(self):
        o = urlparse(self.file_uri)
        # test o.scheme

        filename = os.path.basename(self.local_file_path)
        self.remote_file_path = os.path.join(self.remote_path, filename)

        self._update_state('UploadRunning')  # lets start the upload
        if self.debug:
            print('[file] starting upload of %s' % filename)

        try:
            self.bytes_sent = 0.0
            self.bytes_total = os.path.getsize(self.local_file_path)
            f = open(self.local_file_path, 'r')
        except OSError as e:
            self._update_state('Error')
            self._update_error('file', str(e))
            return

        try:
            self.progress = 0.0
            ftp = ftplib.FTP()
            ftp.connect(host=o.hostname, port=o.port)
            ftp.login()
            ftp.storbinary(
                'STOR %s' % filename,
                f,
                blocksize=8192,
                callback=self._progress_callback,
            )
            ftp.close()
            f.close()
        except Exception as e:
            self._update_state('Error')
            self._update_error('ftp', str(e))
            return

        self._update_state('NoTransfer')  # upload successfully finished
        if self.debug:
            print('[file] upload of %s finished' % filename)

    def _download_worker(self):
        o = urlparse(self.file_uri)
        # test o.scheme

        filename = self.remote_file_path[len(self.remote_path) :]  # mid
        self.local_file_path = os.path.join(self.local_path, filename)

        self._update_state('DownloadRunning')  # lets start the upload
        if self.debug:
            print('[file] starting download of %s' % filename)

        try:
            local_path = os.path.dirname(os.path.abspath(self.local_file_path))
            if not os.path.exists(local_path):
                os.makedirs(local_path)
            self.file = open(self.local_file_path, 'w')
        except Exception as e:
            self._update_state('Error')
            self._update_error('file', str(e))
            return

        try:
            ftp = ftplib.FTP()
            ftp.connect(host=o.hostname, port=o.port)
            ftp.login()
            ftp.sendcmd("TYPE i")  # Switch to Binary mode
            self.progress = 0.0
            self.bytes_sent = 0.0
            self.bytes_total = ftp.size(filename)
            ftp.retrbinary('RETR %s' % filename, self._progress_callback)
            ftp.close()
            self.file.close()
            self.file = None
        except Exception as e:
            self._update_state('Error')
            self._update_error('ftp', str(e))
            return

        self._update_state('NoTransfer')  # upload successfully finished
        if self.debug:
            print('[file] download of %s finished' % filename)

    def _refresh_files_worker(self):
        o = urlparse(self.file_uri)
        # test o.scheme

        self._update_state('RefreshRunning')  # lets start the upload
        if self.debug:
            print('[file] starting file list refresh')

        try:
            ftp = ftplib.FTP()
            ftp.connect(host=o.hostname, port=o.port)
            ftp.login()
            with self.file_list_lock:
                self._file_list = ftp.nlst()
            ftp.close()
        except Exception as e:
            self._update_state('Error')
            self._update_error('ftp', str(e))
            return

        self._update_state('NoTransfer')  # upload successfully finished
        if self.debug:
            print('[file] file refresh finished')

    def _remove_file_worker(self, filename):
        o = urlparse(self.file_uri)
        # test o.scheme

        self._update_state('RemoveRunning')  # lets start the upload
        if self.debug:
            print('[file] removing %s' % filename)

        try:
            ftp = ftplib.FTP()
            ftp.connect(host=o.hostname, port=o.port)
            ftp.login()
            ftp.delete(filename)
            ftp.close()
        except Exception as e:
            self._update_state('Error')
            self._update_error('ftp', str(e))
            return

        self._update_state('NoTransfer')  # upload successfully finished
        if self.debug:
            print('[file] removing %s completed' % filename)

    def _progress_callback(self, data):
        if self.file is not None:
            self.file.write(data)
        self.bytes_sent += 8192
        self.progress = self.bytes_sent / self.bytes_total

    def start_upload(self):
        with self.state_condition:
            if not self.ready or self.transfer_state != 'NoTransfer':
                return

        thread = threading.Thread(target=self._upload_worker)
        thread.start()

    def start_download(self):
        with self.state_condition:
            if not self.ready or self.transfer_state != 'NoTransfer':
                return

        thread = threading.Thread(target=self._download_worker)
        thread.start()

    def refresh_files(self):
        with self.state_condition:
            if not self.ready or self.transfer_state != 'NoTransfer':
                return

        thread = threading.Thread(target=self._refresh_files_worker)
        thread.start()

    def remove_file(self, name):
        with self.state_condition:
            if not self.ready or self.transfer_state != 'NoTransfer':
                return

        thread = threading.Thread(target=self._remove_file_worker, args=(name,))
        thread.start()

    def abort(self):
        raise NotImplementedError('not implemented')

    def wait_completed(self, timeout=None):
        with self.state_condition:
            if self.transfer_state == 'NoTransfer':
                return True
            if self.transfer_state == 'Error':
                return False
            self.state_condition.wait(timeout=timeout)
            return self.transfer_state == 'NoTransfer'

    def _update_state(self, state):
        with self.state_condition:
            if self.transfer_state != state:
                self.transfer_state = state
                self.state_condition.notify()

    def _update_error(self, error, description):
        self.error_string = '[file] error: %s %s' % (error, description)

    def clear_error(self):
        self.error_string = ''

    def start(self):
        pass

    def stop(self):
        pass



# coding=utf-8
from fysom import Fysom
from ..common.publish import Publish

import machinetalk.protobuf.types_pb2 as pb
from machinetalk.protobuf.message_pb2 import Container


class LogServiceBase(object):
    def __init__(self, debuglevel=0, debugname='Log Service Base'):
        self.debuglevel = debuglevel
        self.debugname = debugname
        self._error_string = ''
        self.on_error_string_changed = []

        # Log
        self._log_channel = Publish(debuglevel=debuglevel)
        self._log_channel.debugname = '%s - %s' % (self.debugname, 'log')
        # more efficient to reuse protobuf messages
        self._log_tx = Container()

        # callbacks
        self.on_state_changed = []

        # fsm
        self._fsm = Fysom(
            {
                'initial': 'down',
                'events': [
                    {'name': 'connect', 'src': 'down', 'dst': 'up'},
                    {'name': 'disconnect', 'src': 'up', 'dst': 'down'},
                ],
            }
        )

        self._fsm.ondown = self._on_fsm_down
        self._fsm.onafterconnect = self._on_fsm_connect
        self._fsm.onup = self._on_fsm_up
        self._fsm.onafterdisconnect = self._on_fsm_disconnect

    def _on_fsm_down(self, _):
        if self.debuglevel > 0:
            print('[%s]: state DOWN' % self.debugname)
        for cb in self.on_state_changed:
            cb('down')
        return True

    def _on_fsm_connect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event CONNECT' % self.debugname)
        self.start_log_channel()
        return True

    def _on_fsm_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: state UP' % self.debugname)
        for cb in self.on_state_changed:
            cb('up')
        return True

    def _on_fsm_disconnect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event DISCONNECT' % self.debugname)
        self.stop_log_channel()
        return True

    @property
    def error_string(self):
        return self._error_string

    @error_string.setter
    def error_string(self, string):
        if self._error_string is string:
            return
        self._error_string = string
        for cb in self.on_error_string_changed:
            cb(string)

    @property
    def log_uri(self):
        return self._log_channel.socket_uri

    @log_uri.setter
    def log_uri(self, value):
        self._log_channel.socket_uri = value

    @property
    def log_port(self):
        return self._log_channel.socket_port

    @property
    def log_dsn(self):
        return self._log_channel.socket_dsn

    def start(self):
        if self._fsm.isstate('down'):
            self._fsm.connect()

    def stop(self):
        if self._fsm.isstate('up'):
            self._fsm.disconnect()

    def add_log_topic(self, name):
        self._log_channel.add_socket_topic(name)

    def remove_log_topic(self, name):
        self._log_channel.remove_socket_topic(name)

    def clear_log_topics(self):
        self._log_channel.clear_socket_topics()

    def start_log_channel(self):
        self._log_channel.start()

    def stop_log_channel(self):
        self._log_channel.stop()

    def send_log_message(self, identity, msg_type, tx):
        self._log_channel.send_socket_message(identity, msg_type, tx)

    def send_log_message(self, identity, tx):
        ids = [identity]
        for receiver in ids:
            self.send_log_message(receiver, pb.MT_LOG_MESSAGE, tx)



# coding=utf-8
from fysom import Fysom
from ..application.errorsubscribe import ErrorSubscribe

import machinetalk.protobuf.types_pb2 as pb
from machinetalk.protobuf.message_pb2 import Container


class ErrorBase(object):
    def __init__(self, debuglevel=0, debugname='Error Base'):
        self.debuglevel = debuglevel
        self.debugname = debugname
        self._error_string = ''
        self.on_error_string_changed = []

        # Error
        self._error_channel = ErrorSubscribe(debuglevel=debuglevel)
        self._error_channel.debugname = '%s - %s' % (self.debugname, 'error')
        self._error_channel.on_state_changed.append(self._error_channel_state_changed)
        self._error_channel.on_socket_message_received.append(
            self._error_channel_message_received
        )
        # more efficient to reuse protobuf messages
        self._error_rx = Container()

        # callbacks
        self.on_error_message_received = []
        self.on_state_changed = []

        # fsm
        self._fsm = Fysom(
            {
                'initial': 'down',
                'events': [
                    {'name': 'connect', 'src': 'down', 'dst': 'trying'},
                    {'name': 'error_up', 'src': 'trying', 'dst': 'up'},
                    {'name': 'disconnect', 'src': 'trying', 'dst': 'down'},
                    {'name': 'error_trying', 'src': 'up', 'dst': 'trying'},
                    {'name': 'disconnect', 'src': 'up', 'dst': 'down'},
                ],
            }
        )

        self._fsm.ondown = self._on_fsm_down
        self._fsm.onafterconnect = self._on_fsm_connect
        self._fsm.ontrying = self._on_fsm_trying
        self._fsm.onaftererror_up = self._on_fsm_error_up
        self._fsm.onafterdisconnect = self._on_fsm_disconnect
        self._fsm.onup = self._on_fsm_up
        self._fsm.onaftererror_trying = self._on_fsm_error_trying
        self._fsm.onleaveup = self._on_fsm_up_exit

    def _on_fsm_down(self, _):
        if self.debuglevel > 0:
            print('[%s]: state DOWN' % self.debugname)
        for cb in self.on_state_changed:
            cb('down')
        return True

    def _on_fsm_connect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event CONNECT' % self.debugname)
        self.update_topics()
        self.start_error_channel()
        return True

    def _on_fsm_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: state TRYING' % self.debugname)
        for cb in self.on_state_changed:
            cb('trying')
        return True

    def _on_fsm_error_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: event ERROR UP' % self.debugname)
        return True

    def _on_fsm_disconnect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event DISCONNECT' % self.debugname)
        self.stop_error_channel()
        return True

    def _on_fsm_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: state UP entry' % self.debugname)
        self.set_connected()
        if self.debuglevel > 0:
            print('[%s]: state UP' % self.debugname)
        for cb in self.on_state_changed:
            cb('up')
        return True

    def _on_fsm_error_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: event ERROR TRYING' % self.debugname)
        return True

    def _on_fsm_up_exit(self, _):
        if self.debuglevel > 0:
            print('[%s]: state UP exit' % self.debugname)
        self.clear_connected()
        return True

    @property
    def error_string(self):
        return self._error_string

    @error_string.setter
    def error_string(self, string):
        if self._error_string is string:
            return
        self._error_string = string
        for cb in self.on_error_string_changed:
            cb(string)

    @property
    def error_uri(self):
        return self._error_channel.socket_uri

    @error_uri.setter
    def error_uri(self, value):
        self._error_channel.socket_uri = value

    def update_topics(self):
        print('WARNING: slot update topics unimplemented')

    def set_connected(self):
        print('WARNING: slot set connected unimplemented')

    def clear_connected(self):
        print('WARNING: slot clear connected unimplemented')

    def start(self):
        if self._fsm.isstate('down'):
            self._fsm.connect()

    def stop(self):
        if self._fsm.isstate('trying'):
            self._fsm.disconnect()
        elif self._fsm.isstate('up'):
            self._fsm.disconnect()

    def add_error_topic(self, name):
        self._error_channel.add_socket_topic(name)

    def remove_error_topic(self, name):
        self._error_channel.remove_socket_topic(name)

    def clear_error_topics(self):
        self._error_channel.clear_socket_topics()

    def start_error_channel(self):
        self._error_channel.start()

    def stop_error_channel(self):
        self._error_channel.stop()

    # process all messages received on error
    def _error_channel_message_received(self, identity, rx):

        # react to emc nml error message
        if rx.type == pb.MT_EMC_NML_ERROR:
            self.emc_nml_error_received(identity, rx)

        # react to emc nml text message
        elif rx.type == pb.MT_EMC_NML_TEXT:
            self.emc_nml_text_received(identity, rx)

        # react to emc nml display message
        elif rx.type == pb.MT_EMC_NML_DISPLAY:
            self.emc_nml_display_received(identity, rx)

        # react to emc operator text message
        elif rx.type == pb.MT_EMC_OPERATOR_TEXT:
            self.emc_operator_text_received(identity, rx)

        # react to emc operator error message
        elif rx.type == pb.MT_EMC_OPERATOR_ERROR:
            self.emc_operator_error_received(identity, rx)

        # react to emc operator display message
        elif rx.type == pb.MT_EMC_OPERATOR_DISPLAY:
            self.emc_operator_display_received(identity, rx)

        for cb in self.on_error_message_received:
            cb(identity, rx)

    def emc_nml_error_received(self, identity, rx):
        print('SLOT emc nml error unimplemented')

    def emc_nml_text_received(self, identity, rx):
        print('SLOT emc nml text unimplemented')

    def emc_nml_display_received(self, identity, rx):
        print('SLOT emc nml display unimplemented')

    def emc_operator_text_received(self, identity, rx):
        print('SLOT emc operator text unimplemented')

    def emc_operator_error_received(self, identity, rx):
        print('SLOT emc operator error unimplemented')

    def emc_operator_display_received(self, identity, rx):
        print('SLOT emc operator display unimplemented')

    def _error_channel_state_changed(self, state):

        if state == 'trying':
            if self._fsm.isstate('up'):
                self._fsm.error_trying()

        elif state == 'up':
            if self._fsm.isstate('trying'):
                self._fsm.error_up()



# coding=utf-8
import threading
from collections import namedtuple

from ..common import ComponentBase
from ..dns_sd import ServiceContainer, Service
from ..machinetalk_core.application.logbase import LogBase

# noinspection PyUnresolvedReferences
from machinetalk.protobuf.types_pb2 import (
    RTAPI_MSG_ALL,
    RTAPI_MSG_DBG,
    RTAPI_MSG_ERR,
    RTAPI_MSG_INFO,
    RTAPI_MSG_WARN,
    MSG_KERNEL,
    MSG_RTUSER,
    MSG_ULAPI,
)


ApplicationLogMessage = namedtuple(
    'ApplicationLogMessage', 'level origin tag pid text timestamp'
)


class ApplicationLog(ComponentBase, LogBase, ServiceContainer):
    def __init__(self, debug=False):
        LogBase.__init__(self, debuglevel=int(debug))
        ComponentBase.__init__(self)
        ServiceContainer.__init__(self)
        self.connected_condition = threading.Condition(threading.Lock())
        self.debug = debug

        # callbacks
        self.on_connected_changed = []
        self.on_message_received = []

        self.connected = False
        self.log_level = RTAPI_MSG_ALL

        self._log_service = Service(type_='log')
        self.add_service(self._log_service)
        self.on_services_ready_changed.append(self._on_services_ready_changed)

    # slot
    def update_topics(self):
        self.clear_log_topics()
        self.add_log_topic('log')

    # slot
    def set_connected(self):
        self._update_connected(True)

    # slot
    def clear_connected(self):
        self._update_connected(False)

    # slot
    def log_message_received(self, identity, rx):
        log_message = rx.log_message
        print('received {}'.format(log_message))
        if log_message.level > self.log_level:
            return

        msg = ApplicationLogMessage(
            level=log_message.level,
            origin=log_message.origin,
            pid=log_message.pid,
            tag=log_message.tag,
            text=log_message.text,
            timestamp=self._convert_timestamp(rx.tv_sec, rx.tv_nsec),
        )
        for cb in self.on_message_received:
            cb(msg)

    @staticmethod
    def _convert_timestamp(sec, nsec):
        return sec * 1000 + nsec / 1000000

    def _update_connected(self, connected):
        with self.connected_condition:
            self.connected = connected
            self.connected_condition.notify()
        for cb in self.on_connected_changed:
            cb(connected)

    def _on_services_ready_changed(self, ready):
        self.log_uri = self._log_service.uri
        self.ready = ready



#!/usr/bin/env python
# coding=utf-8
from pymachinetalk.application import ApplicationCommand, EMC_TASK_STATE_ESTOP_RESET, EMC_TASK_STATE_OFF

from pymachinetalk.dns_sd import ServiceDiscovery

MAX_WAIT_CONNECTED = 5.0
COMMAND_TIMEOUT = 0.5
DEBUG = False


def print_state(state):
    print('command state: {}'.format(state))


def get_out_of_estop():
    sd = ServiceDiscovery()
    command = ApplicationCommand(debug=DEBUG)
    if DEBUG:
        command.on_state_changed.append(print_state)
    sd.register(command)
    sd.start()

    connected = command.wait_connected(timeout=MAX_WAIT_CONNECTED)
    if not connected:
        raise RuntimeError('Could not connect')
    else:
        print('connected')

    print('executing ESTOP reset')
    ticket = command.set_task_state(EMC_TASK_STATE_ESTOP_RESET)
    if not command.wait_completed(ticket=ticket, timeout=COMMAND_TIMEOUT):
        raise RuntimeError('Task did not complete')
    else:
        print('done')

    print('executing TASK STATE ON')
    command.set_task_state(EMC_TASK_STATE_OFF)
    if not command.wait_completed(timeout=COMMAND_TIMEOUT):
        raise RuntimeError('Task did not complete')
    else:
        print('done')


if __name__ == '__main__':
    get_out_of_estop()



# coding=utf-8
from fysom import Fysom
from ..common.rpcclient import RpcClient
from ..application.launchersubscribe import LauncherSubscribe

import machinetalk.protobuf.types_pb2 as pb
from machinetalk.protobuf.message_pb2 import Container


class LauncherBase(object):
    def __init__(self, debuglevel=0, debugname='Launcher Base'):
        self.debuglevel = debuglevel
        self.debugname = debugname
        self._error_string = ''
        self.on_error_string_changed = []

        # Launchercmd
        self._launchercmd_channel = RpcClient(debuglevel=debuglevel)
        self._launchercmd_channel.debugname = '%s - %s' % (
            self.debugname,
            'launchercmd',
        )
        self._launchercmd_channel.on_state_changed.append(
            self._launchercmd_channel_state_changed
        )
        self._launchercmd_channel.on_socket_message_received.append(
            self._launchercmd_channel_message_received
        )
        # more efficient to reuse protobuf messages
        self._launchercmd_rx = Container()
        self._launchercmd_tx = Container()

        # Launcher
        self._launcher_channel = LauncherSubscribe(debuglevel=debuglevel)
        self._launcher_channel.debugname = '%s - %s' % (self.debugname, 'launcher')
        self._launcher_channel.on_state_changed.append(
            self._launcher_channel_state_changed
        )
        self._launcher_channel.on_socket_message_received.append(
            self._launcher_channel_message_received
        )
        # more efficient to reuse protobuf messages
        self._launcher_rx = Container()

        # callbacks
        self.on_launchercmd_message_received = []
        self.on_launcher_message_received = []
        self.on_state_changed = []

        # fsm
        self._fsm = Fysom(
            {
                'initial': 'down',
                'events': [
                    {'name': 'connect', 'src': 'down', 'dst': 'trying'},
                    {'name': 'launchercmd_up', 'src': 'trying', 'dst': 'syncing'},
                    {'name': 'disconnect', 'src': 'trying', 'dst': 'down'},
                    {'name': 'launchercmd_trying', 'src': 'syncing', 'dst': 'trying'},
                    {'name': 'launcher_up', 'src': 'syncing', 'dst': 'synced'},
                    {'name': 'disconnect', 'src': 'syncing', 'dst': 'down'},
                    {'name': 'launcher_trying', 'src': 'synced', 'dst': 'syncing'},
                    {'name': 'launchercmd_trying', 'src': 'synced', 'dst': 'trying'},
                    {'name': 'disconnect', 'src': 'synced', 'dst': 'down'},
                ],
            }
        )

        self._fsm.ondown = self._on_fsm_down
        self._fsm.onafterconnect = self._on_fsm_connect
        self._fsm.ontrying = self._on_fsm_trying
        self._fsm.onafterlaunchercmd_up = self._on_fsm_launchercmd_up
        self._fsm.onafterdisconnect = self._on_fsm_disconnect
        self._fsm.onsyncing = self._on_fsm_syncing
        self._fsm.onafterlaunchercmd_trying = self._on_fsm_launchercmd_trying
        self._fsm.onafterlauncher_up = self._on_fsm_launcher_up
        self._fsm.onsynced = self._on_fsm_synced
        self._fsm.onafterlauncher_trying = self._on_fsm_launcher_trying
        self._fsm.onleavesynced = self._on_fsm_synced_exit

    def _on_fsm_down(self, _):
        if self.debuglevel > 0:
            print('[%s]: state DOWN' % self.debugname)
        for cb in self.on_state_changed:
            cb('down')
        return True

    def _on_fsm_connect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event CONNECT' % self.debugname)
        self.start_launchercmd_channel()
        return True

    def _on_fsm_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: state TRYING' % self.debugname)
        for cb in self.on_state_changed:
            cb('trying')
        return True

    def _on_fsm_launchercmd_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: event LAUNCHERCMD UP' % self.debugname)
        self.start_launcher_channel()
        return True

    def _on_fsm_disconnect(self, _):
        if self.debuglevel > 0:
            print('[%s]: event DISCONNECT' % self.debugname)
        self.stop_launchercmd_channel()
        self.stop_launcher_channel()
        return True

    def _on_fsm_syncing(self, _):
        if self.debuglevel > 0:
            print('[%s]: state SYNCING' % self.debugname)
        for cb in self.on_state_changed:
            cb('syncing')
        return True

    def _on_fsm_launchercmd_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: event LAUNCHERCMD TRYING' % self.debugname)
        self.stop_launcher_channel()
        return True

    def _on_fsm_launcher_up(self, _):
        if self.debuglevel > 0:
            print('[%s]: event LAUNCHER UP' % self.debugname)
        return True

    def _on_fsm_synced(self, _):
        if self.debuglevel > 0:
            print('[%s]: state SYNCED entry' % self.debugname)
        self.sync_status()
        if self.debuglevel > 0:
            print('[%s]: state SYNCED' % self.debugname)
        for cb in self.on_state_changed:
            cb('synced')
        return True

    def _on_fsm_launcher_trying(self, _):
        if self.debuglevel > 0:
            print('[%s]: event LAUNCHER TRYING' % self.debugname)
        return True

    def _on_fsm_synced_exit(self, _):
        if self.debuglevel > 0:
            print('[%s]: state SYNCED exit' % self.debugname)
        self.unsync_status()
        return True

    @property
    def error_string(self):
        return self._error_string

    @error_string.setter
    def error_string(self, string):
        if self._error_string is string:
            return
        self._error_string = string
        for cb in self.on_error_string_changed:
            cb(string)

    @property
    def launchercmd_uri(self):
        return self._launchercmd_channel.socket_uri

    @launchercmd_uri.setter
    def launchercmd_uri(self, value):
        self._launchercmd_channel.socket_uri = value

    @property
    def launcher_uri(self):
        return self._launcher_channel.socket_uri

    @launcher_uri.setter
    def launcher_uri(self, value):
        self._launcher_channel.socket_uri = value

    def sync_status(self):
        print('WARNING: slot sync status unimplemented')

    def unsync_status(self):
        print('WARNING: slot unsync status unimplemented')

    def start(self):
        if self._fsm.isstate('down'):
            self._fsm.connect()

    def stop(self):
        if self._fsm.isstate('trying'):
            self._fsm.disconnect()
        elif self._fsm.isstate('syncing'):
            self._fsm.disconnect()
        elif self._fsm.isstate('synced'):
            self._fsm.disconnect()

    def add_launcher_topic(self, name):
        self._launcher_channel.add_socket_topic(name)

    def remove_launcher_topic(self, name):
        self._launcher_channel.remove_socket_topic(name)

    def clear_launcher_topics(self):
        self._launcher_channel.clear_socket_topics()

    def start_launchercmd_channel(self):
        self._launchercmd_channel.start()

    def stop_launchercmd_channel(self):
        self._launchercmd_channel.stop()

    def start_launcher_channel(self):
        self._launcher_channel.start()

    def stop_launcher_channel(self):
        self._launcher_channel.stop()

    # process all messages received on launchercmd
    def _launchercmd_channel_message_received(self, rx):

        # react to error message
        if rx.type == pb.MT_ERROR:
            # update error string with note
            self.error_string = ''
            for note in rx.note:
                self.error_string += note + '\n'

        for cb in self.on_launchercmd_message_received:
            cb(rx)

    # process all messages received on launcher
    def _launcher_channel_message_received(self, identity, rx):

        # react to launcher full update message
        if rx.type == pb.MT_LAUNCHER_FULL_UPDATE:
            self.launcher_full_update_received(identity, rx)

        # react to launcher incremental update message
        elif rx.type == pb.MT_LAUNCHER_INCREMENTAL_UPDATE:
            self.launcher_incremental_update_received(identity, rx)

        for cb in self.on_launcher_message_received:
            cb(identity, rx)

    def launcher_full_update_received(self, identity, rx):
        print('SLOT launcher full update unimplemented')

    def launcher_incremental_update_received(self, identity, rx):
        print('SLOT launcher incremental update unimplemented')

    def send_launchercmd_message(self, msg_type, tx):
        self._launchercmd_channel.send_socket_message(msg_type, tx)

    def send_launcher_start(self, tx):
        self.send_launchercmd_message(pb.MT_LAUNCHER_START, tx)

    def send_launcher_kill(self, tx):
        self.send_launchercmd_message(pb.MT_LAUNCHER_KILL, tx)

    def send_launcher_terminate(self, tx):
        self.send_launchercmd_message(pb.MT_LAUNCHER_TERMINATE, tx)

    def send_launcher_write_stdin(self, tx):
        self.send_launchercmd_message(pb.MT_LAUNCHER_WRITE_STDIN, tx)

    def send_launcher_call(self, tx):
        self.send_launchercmd_message(pb.MT_LAUNCHER_CALL, tx)

    def send_launcher_shutdown(self, tx):
        self.send_launchercmd_message(pb.MT_LAUNCHER_SHUTDOWN, tx)

    def send_launcher_set(self, tx):
        self.send_launchercmd_message(pb.MT_LAUNCHER_SET, tx)

    def _launchercmd_channel_state_changed(self, state):

        if state == 'trying':
            if self._fsm.isstate('syncing'):
                self._fsm.launchercmd_trying()
            elif self._fsm.isstate('synced'):
                self._fsm.launchercmd_trying()

        elif state == 'up':
            if self._fsm.isstate('trying'):
                self._fsm.launchercmd_up()

    def _launcher_channel_state_changed(self, state):

        if state == 'trying':
            if self._fsm.isstate('synced'):
                self._fsm.launcher_trying()

        elif state == 'up':
            if self._fsm.isstate('syncing'):
                self._fsm.launcher_up()



from __future__ import unicode_literals
import socket
from zeroconf import ServiceBrowser, Zeroconf, ServiceInfo
import six
from six.moves.urllib.parse import urlparse


class Service(object):
    def __init__(self, type_=''):
        self.type = type_
        self.domain = 'local'
        self.base_type = 'machinekit'
        self.protocol = 'tcp'
        self.name = ''
        self.uri = ''
        self.uuid = ''
        self.host_name = ''
        self.host_address = ''
        self.version = 0
        self._raw_uri = ''
        self._ready = False

        self.service_infos = []

        # callback
        self.on_ready_changed = []

    @property
    def ready(self):
        return self._ready

    @ready.setter
    def ready(self, value):
        if value != self._ready:
            self._ready = value
            for cb in self.on_ready_changed:
                cb(value)

    @property
    def typestring(self):
        return '_%s._%s.%s.' % (self.base_type, self.protocol, self.domain)

    def matches_service_info(self, info):
        return (
            self.type == info.properties.get(b'service', b'').decode()
            and self.typestring in info.type
        )

    def __eq__(self, other):
        if isinstance(other, ServiceInfo):
            return self.name == other.name
        return False

    def add_service_info(self, info):
        self.service_infos.append(info)
        self._update()

    def remove_service_info(self, _):
        for info in self.service_infos:
            if self == info:
                self.service_infos.remove(info)
                break
        self._update()

    def clear_service_infos(self):
        self.service_infos = []
        self._update()

    def _update(self):
        if any(self.service_infos):
            info = self.service_infos[0]
            self._set_all_values_from_service_info(info)
            self.ready = True
        else:
            self.ready = False
            self._init_all_values()

    def _set_all_values_from_service_info(self, info):
        self.name = info.name
        self._raw_uri = info.properties.get(b'dsn', b'').decode()
        self.uuid = info.properties.get(b'uuid', b'').decode()
        self.version = info.properties.get(b'version', b'')
        self.host_name = info.server
        try:
            self.host_address = socket.inet_ntoa(info.address).decode()
        except (OSError, socket.error):
            self.host_address = info.address.decode()
        self._update_uri()

    def _update_uri(self):
        url = urlparse(self._raw_uri)
        host = url.hostname
        if (
            not (host is None or self.host_name is None)
            and host.lower() in self.host_name.lower()
        ):  # hostname is in form .local. and host in .local
            netloc = url.netloc
            netloc = netloc.replace(host, self.host_address)
            new_url = url._replace(netloc=netloc)  # use resolved address
            self.uri = new_url.geturl()
        else:
            self.uri = self._raw_uri  # pass raw uri

    def _init_all_values(self):
        self.name = ''
        self.uri = ''
        self.uuid = ''
        self.host_name = ''
        self.host_address = ''
        self.version = 0


class ServiceDiscoveryFilter(object):
    def __init__(self, name='', txt_records=None):
        if txt_records is None:
            txt_records = {}
        self.name = name
        self.txt_records = txt_records

    def matches_service_info(self, info):
        if not isinstance(info, ServiceInfo):
            raise TypeError('must pass a ServiceInfo object')
        match = True
        if self.name not in info.name:
            match = False
        for name, value in six.iteritems(self.txt_records):
            if not info.properties[name] == value:
                match = False
                break
        return match


class ServiceDiscovery(object):
    def __init__(
        self,
        service_type='machinekit',
        filter_=ServiceDiscoveryFilter(),
        nameservers=None,
        lookup_interval=None,
    ):
        """ Initialize the multicast or unicast DNS-SD service discovery instance.
        @param service_type DNS-SD type use for discovery, does not need to be changed for Machinekit.
        @param filter_ Optional filter can be used to look for specific instances.
        @param nameservers Pass one or more nameserver addresses to enabled unicast service discovery.
        @param lookup_interval How often the SD should send out service queries.
        """
        if nameservers is None:
            nameservers = []
        self.service_type = service_type
        self.filter = filter_
        self.nameservers = nameservers
        self.lookup_interval = lookup_interval

        self.is_ready = False
        self.services = []
        self._browsers = []
        self._zeroconfs = []

    def _start_discovery(self):
        self._zeroconfs = []
        self._browsers = []
        if any(self.nameservers):
            self._start_unicast_discovery()
        else:
            self._start_multicast_discovery()

    def _start_multicast_discovery(self):
        type_string = '_%s._tcp.local.' % self.service_type
        zeroconf = Zeroconf()
        self._zeroconfs.append(zeroconf)
        kwargs = {}
        if self.lookup_interval:
            kwargs['delay'] = self.lookup_interval
        self._browsers.append(ServiceBrowser(zeroconf, type_string, self, **kwargs))

    def _start_unicast_discovery(self):
        for service in self.services:
            type_string = '_%s._sub._%s._tcp.local.' % (service.type, self.service_type)
            zeroconf = Zeroconf(unicast=True)
            self._zeroconfs.append(zeroconf)
            for nameserver in self.nameservers:
                kwargs = {'addr': nameserver}
                if self.lookup_interval:
                    kwargs['delay'] = self.lookup_interval
                self._browsers.append(
                    ServiceBrowser(zeroconf, type_string, self, **kwargs)
                )

    def _stop_discovery(self):
        for zeroconf in self._zeroconfs:
            zeroconf.close()
        del self._zeroconfs[:]
        del self._browsers[:]
        for service in self.services:
            service.clear_service_infos()

    def remove_service(self, zeroconf, type_, name):
        info = zeroconf.get_service_info(type_, name)
        if info is None:
            return
        for service in self.services:
            if self.filter.matches_service_info(info) and service.matches_service_info(
                info
            ):
                service.remove_service_info(info)

    def add_service(self, zeroconf, type_, name):
        info = zeroconf.get_service_info(type_, name)
        if info is None:
            return
        for service in self.services:
            if self.filter.matches_service_info(info) and service.matches_service_info(
                info
            ):
                service.add_service_info(info)

    @staticmethod
    def _verify_item_and_run(item, cmd):
        if isinstance(item, ServiceContainer):
            for service in item.services:
                cmd(service)
        elif isinstance(item, Service):
            cmd(item)
        else:
            raise TypeError('passed unregisterable item')

    def register(self, item):
        if self.is_ready:
            raise RuntimeError(
                'cannot register service when service discovery is already running'
            )
        self._verify_item_and_run(item, self.services.append)

    def unregister(self, item):
        if self.is_ready:
            raise RuntimeError(
                'cannot unregister service when service discovery is already running'
            )
        self._verify_item_and_run(item, self.services.remove)

    def start(self):
        if not self._browsers:
            self.is_ready = True
            self._start_discovery()

    def stop(self):
        self.is_ready = False
        self._stop_discovery()


class ServiceContainer(object):
    def __init__(self):
        self._services = []
        self._services_ready = False

        self.on_services_ready_changed = []

    @property
    def services(self):
        return self._services

    def add_service(self, service):
        if not isinstance(service, Service):
            raise TypeError('only Service is supported')
        self._services.append(service)
        service.on_ready_changed.append(self._update_services_ready)

    def remove_service(self, service):
        if not isinstance(service, Service):
            raise TypeError('only Service is supported')
        self._services.remove(service)
        service.on_ready_changed.remove(self._update_services_ready)

    @property
    def services_ready(self):
        return self._services_ready

    @services_ready.setter
    def services_ready(self, value):
        if value is not self._services_ready:
            self._services_ready = value
            for cb in self.on_services_ready_changed:
                cb(value)

    def _update_services_ready(self, _):
        ready = True
        for service in self._services:
            if not service.ready:
                ready = False
                break
        self.services_ready = ready



#!/usr/bin/env python
import sys
import os
import time
import threading
import zmq

# Machinekit specific, can only use on local machine
from machinekit import config

# Machinetalk bindings
from pymachinetalk.dns_sd import ServiceDiscovery, ServiceDiscoveryFilter
from pymachinetalk.application import ApplicationStatus
from pymachinetalk.application import ApplicationCommand
import pymachinetalk.application as application

from ipcmsg_pb2 import *


if sys.version_info >= (3, 0):
    import configparser
else:
    import ConfigParser as configparser


class IPCServer:
    def __init__(self, uuid, debug=True):
        self.debug = debug
        self.threads = []
        self.shutdown = threading.Event()
        sd_filter = ServiceDiscoveryFilter(txt_records={'uuid': uuid})
        self.sd = ServiceDiscovery(filter_=sd_filter)
        self.status = ApplicationStatus()
        self.command = ApplicationCommand()
        self.sd.register(self.status)
        self.sd.register(self.command)

        # create ipc sockets
        context = zmq.Context()
        context.linger = 0
        self.context = context
        # self.pubSocket = context.socket(zmq.PUB)
        # self.pubSocket.bind('ipc://machinetalk-server.ipc')
        # self.pubDsname = self.pubSocket.get_string(zmq.LAST_ENDPOINT, encoding='utf-8')
        # if self.debug:
        #     print('bound PUB socket to %s' % self.pubDsname)
        self.zmqSocket = context.socket(zmq.ROUTER)
        self.zmqSocket.bind('ipc://machinetalk-server.ipc')
        self.zmqDsname = self.zmqSocket.get_string(zmq.LAST_ENDPOINT, encoding='utf-8')
        if self.debug:
            print('bound ROUTER socket to %s' % self.zmqDsname)
        self.zmqLock = threading.Lock()

        self._tx = Message()
        self._rx = Message()

        self.threads.append(threading.Thread(target=self.socket_worker))
        for thread in self.threads:
            thread.start()

    def send_msg(self, identity, msg_type):
        with self.zmqLock:
            self._tx.type = msg_type
            txBuffer = self._tx.SerializeToString()
            self.zmqSocket.send_multipart([identity, txBuffer], zmq.NOBLOCK)
            self._tx.Clear()

    def socket_worker(self):
        poll = zmq.Poller()
        poll.register(self.zmqSocket, zmq.POLLIN)

        while not self.shutdown.is_set():
            s = dict(poll.poll(200))
            if self.zmqSocket in s and s[self.zmqSocket] == zmq.POLLIN:
                self.process_msg(self.zmqSocket)

    def process_msg(self, socket):
        (identity, message) = socket.recv_multipart()
        self._rx.ParseFromString(message)

        if self.debug:
            print("process message called, id: %s" % identity)
            print(str(self._rx))

        if self._rx.type == IPC_POSITION:
            self._tx.x = (
                self.status.motion.position.x
                - self.status.motion.g5x_offset.x
                - self.status.motion.g92_offset.x
                - self.status.io.tool_offset.x
            )
            self._tx.y = (
                self.status.motion.position.y
                - self.status.motion.g5x_offset.y
                - self.status.motion.g92_offset.y
                - self.status.io.tool_offset.y
            )
            self.send_msg(identity, IPC_POSITION)

        elif self._rx.type == IPC_JOG:
            self.command.set_task_mode(application.EMC_TASK_MODE_MANUAL)
            self.command.jog(
                self._rx.jog_type, self._rx.axis, self._rx.velocity, self._rx.distance
            )

        elif self._rx.type == IPC_CONNECTED:
            self._tx.connected = self.status.synced and self.command.connected
            self.send_msg(identity, IPC_CONNECTED)

    def start(self):
        self.sd.start()

    def stop(self):
        self.sd.stop()
        self.shutdown.set()
        for thread in self.threads:
            thread.join()
        self.threads = []


def main():
    mkconfig = config.Config()
    mkini = os.getenv("MACHINEKIT_INI")
    if mkini is None:
        mkini = mkconfig.MACHINEKIT_INI
    if not os.path.isfile(mkini):
        sys.stderr.write("MACHINEKIT_INI " + mkini + " does not exist\n")
        sys.exit(1)

    mki = configparser.ConfigParser()
    mki.read(mkini)
    uuid = mki.get("MACHINEKIT", "MKUUID")
    # remote = mki.getint("MACHINEKIT", "REMOTE")

    ipcServer = IPCServer(uuid=uuid)
    ipcServer.start()

    try:
        while True:
            time.sleep(0.5)
    except KeyboardInterrupt:
        pass

    print("stopping threads")
    ipcServer.stop()

    # wait for all threads to terminate
    while threading.active_count() > 1:
        time.sleep(0.1)

    print("threads stopped")
    sys.exit(0)


if __name__ == "__main__":
    main()



# coding=utf-8
import pytest
import sys
from pymachinetalk import application
from pymachinetalk import dns_sd


@pytest.mark.skipif(
    sys.version_info >= (3, 0),
    reason="Integration tests hang for some reason with Python3",
)
def test_application_integration():
    status = application.ApplicationStatus()
    command = application.ApplicationCommand()
    error = application.ApplicationError()
    log = application.ApplicationLog()
    appfile = application.ApplicationFile()

    sd = dns_sd.ServiceDiscovery()
    sd.register(status)
    sd.register(command)
    sd.register(error)
    sd.register(log)
    sd.register(appfile)



# coding=utf-8

# protobuf
import machinetalk.protobuf.types_pb2 as types
import machinetalk.protobuf.motcmds_pb2 as motcmds

# noinspection PyUnresolvedReferences
from machinetalk.protobuf.status_pb2 import *

ORIGIN_G54 = types.ORIGIN_G54
ORIGIN_G55 = types.ORIGIN_G55
ORIGIN_G56 = types.ORIGIN_G56
ORIGIN_G57 = types.ORIGIN_G57
ORIGIN_G58 = types.ORIGIN_G58
ORIGIN_G59 = types.ORIGIN_G59
ORIGIN_G59_1 = types.ORIGIN_G59_1
ORIGIN_G59_2 = types.ORIGIN_G59_2
ORIGIN_G59_3 = types.ORIGIN_G59_2

MOTION_UNINITIALIZED = types.UNINITIALIZED_STATUS
MOTION_DONE = types.RCS_DONE
MOTION_EXEC = types.RCS_EXEC
MOTION_ERROR = types.RCS_ERROR
MOTION_RECEIVED = types.RCS_RECEIVED

MOTION_TYPE_NONE = motcmds._EMC_MOTION_TYPE_NONE
MOTION_TYPE_TRAVERSE = motcmds._EMC_MOTION_TYPE_TRAVERSE
MOTION_TYPE_FEED = motcmds._EMC_MOTION_TYPE_FEED
MOTION_TYPE_ARC = motcmds._EMC_MOTION_TYPE_ARC
MOTION_TYPE_TOOLCHANGEE = motcmds._EMC_MOTION_TYPE_TOOLCHANGE
MOTION_TYPE_PROBING = motcmds._EMC_MOTION_TYPE_PROBING
MOTION_TYPE_INDEXROTARY = motcmds._EMC_MOTION_TYPE_INDEXROTARY

RELEASE_BRAKE = 0
ENGAGE_BRAKE = 1

JOG_STOP = 0
JOG_CONTINUOUS = 1
JOG_INCREMENT = 2

SPINDLE_FORWARD = 0
SPINDLE_REVERSE = 1
SPINDLE_OFF = 2
SPINDLE_DECREASE = 3
SPINDLE_INCREASE = 4
SPINDLE_CONSTANT = 5

NML_ERROR = types.MT_EMC_NML_ERROR
NML_TEXT = types.MT_EMC_NML_TEXT
NML_DISPLAY = types.MT_EMC_NML_DISPLAY
OPERATOR_ERROR = types.MT_EMC_OPERATOR_ERROR
OPERATOR_TEXT = types.MT_EMC_OPERATOR_TEXT
OPERATOR_DISPLAY = types.MT_EMC_OPERATOR_DISPLAY



#!/usr/bin/env python
# coding=utf-8

import sys

# We must use setuptools, not distutils, because we need to use the
# namespace_packages option for the "google" package.
try:
    from setuptools import setup, Extension, find_packages
except ImportError:
    try:
        from ez_setup import use_setuptools

        use_setuptools()
        from setuptools import setup, Extension, find_packages
    except ImportError:
        sys.stderr.write(
            "Could not import setuptools; make sure you have setuptools or "
            "ez_setup installed.\n"
        )
        raise

from distutils.command.clean import clean

if sys.version_info[0] == 3:
    # Python 3
    from distutils.command.build_py import build_py_2to3 as build_py
else:
    # Python 2
    from distutils.command.build_py import build_py as build_py

requirements = ['pyzmq', 'protobuf', 'machinetalk-protobuf', 'fysom', 'six']
if sys.version_info <= (3, 3):
    requirements.append('zeroconf<=0.19.1')  # freeze version
else:
    requirements.append('zeroconf')

if __name__ == '__main__':
    setup(
        name="pymachinetalk",
        version="0.11.3",
        description="Python bindings for Machinetalk",
        author="Alexander Roessler",
        author_email="alex@machinekoder.com",
        url="https://github.com/machinekit/pymachinetalk",
        namespace_packages=['pymachinetalk'],
        packages=find_packages(),
        install_requires=requirements,
        extras_require={'dev': ['pytest', 'pytest-mock', 'pytest-pep8', 'pytest-cov']},
        cmdclass={'clean': clean, 'build_py': build_py},
    )



# coding=utf-8
import pytest


@pytest.fixture
def dns_sd():
    from pymachinetalk import dns_sd

    return dns_sd


@pytest.fixture
def sd():
    from pymachinetalk import dns_sd

    sd = dns_sd.ServiceDiscovery()
    return sd


def test_registeringServicesFromServiceContainerWorks(dns_sd, sd):
    service = dns_sd.Service()
    discoverable = dns_sd.ServiceContainer()
    discoverable.services.append(service)

    sd.register(discoverable)

    assert service in sd.services


def test_registeringServiceDirectlyWorks(dns_sd, sd):
    service = dns_sd.Service()

    sd.register(service)

    assert service in sd.services


def test_registeringAnythingElseFails(sd):
    item = object()

    try:
        sd.register(item)
    except TypeError:
        assert True

    assert item not in sd.services


def test_registeringWhenRunningThrowsError(dns_sd, sd):
    service = dns_sd.Service()

    def dummy():
        pass

    sd._start_discovery = dummy
    sd.start()

    try:
        sd.register(service)
    except RuntimeError:
        assert True

    assert service not in sd.services


def test_unregisteringServiceDirectlyWorks(dns_sd, sd):
    service = dns_sd.Service()
    sd.register(service)

    sd.unregister(service)

    assert service not in sd.services


def test_unregisteringServicesFromServiceContainerWorks(dns_sd, sd):
    service = dns_sd.Service()
    discoverable = dns_sd.ServiceContainer()
    discoverable.services.append(service)
    sd.register(discoverable)

    sd.unregister(discoverable)

    assert service not in sd.services


def test_unregisteringAnythingElseFails(sd):
    item = 34

    try:
        sd.unregister(item)
    except TypeError:
        assert True

    assert item not in sd.services


def test_unregisteringWhenRunningThrowsError(dns_sd, sd):
    service = dns_sd.Service()

    def dummy():
        pass

    sd._start_discovery = dummy
    sd.start()

    try:
        sd.unregister(service)
    except RuntimeError:
        assert True

    assert service not in sd.services


class ServiceInfoFactory(object):
    def create(
        self,
        base_type='machinekit',
        domain='local',
        sd_protocol='tcp',
        name='Hugo on Franz',
        service=b'halrcomp',
        uuid=b'12345678',
        host='127.0.0.1',
        protocol='tcp',
        port=12345,
        version=0,
        properties=None,
        server='127.0.0.1',
        address=None,
    ):
        from zeroconf import ServiceInfo

        typestring = '_%s._%s.%s.' % (base_type, sd_protocol, domain)
        dsn = b'%s://%s:%i' % (protocol.encode(), host.encode(), port)
        if properties is None:
            properties = {
                b'uuid': uuid,
                b'service': service,
                b'dsn': dsn,
                b'version': version,
            }
        return ServiceInfo(
            type_=typestring,
            name='%s %s.%s' % (name, host, typestring),
            properties=properties,
            address=(address or host).encode(),
            port=port,
            server=server,
        )


@pytest.fixture
def zeroconf(mocker):
    from zeroconf import Zeroconf

    service_info = ServiceInfoFactory().create()
    zeroconf_stub = mocker.stub(name='get_service_info')
    zeroconf_stub.return_value = service_info
    stub_object = Zeroconf()
    stub_object.get_service_info = zeroconf_stub
    return stub_object


@pytest.fixture
def zeroconf_without_service_info(mocker):
    from zeroconf import Zeroconf

    zeroconf_stub = mocker.stub(name='get_service_info')
    zeroconf_stub.return_value = None
    stub_object = Zeroconf()
    stub_object.get_service_info = zeroconf_stub
    return stub_object


def test_serviceDiscoveredUpdatesRegisteredServices(dns_sd, sd, zeroconf):
    service = dns_sd.Service(type_='halrcomp')
    sd.register(service)

    sd.add_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'Foo on Bar 127.0.0.1._machinekit._tcp.local.',
    )

    assert service.ready is True


def test_serviceDisappearedUpdatesRegisteredServices(dns_sd, sd, zeroconf):
    service = dns_sd.Service(type_='halrcomp')
    sd.register(service)

    sd.add_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'Foo on Bar 127.0.0.1._machinekit._tcp.local.',
    )
    sd.remove_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'Foo on Bar 127.0.0.1._machinekit._tcp.local.',
    )

    assert service.ready is False


def test_stoppingServiceDiscoveryResetsAllServices(dns_sd, sd, zeroconf):
    service1 = dns_sd.Service(type_='halrcomp')
    sd.register(service1)
    service2 = dns_sd.Service(type_='halrcmd')
    sd.register(service2)
    sd.browser = object()  # dummy
    sd.add_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'Foo on Bar 127.0.0.1._machinekit._tcp.local.',
    )

    sd.stop()

    assert service1.ready is False
    assert service2.ready is False


def test_serviceDiscoveredWithoutServiceInfoDoesNotUpdateRegisteredServices(
    dns_sd, sd, zeroconf_without_service_info
):
    service = dns_sd.Service(type_='halrcomp')
    sd.register(service)

    sd.add_service(
        zeroconf_without_service_info,
        '_machinekit._tcp.local.',
        'Foo on Bar 127.0.0.1._machinekit._tcp.local.',
    )

    assert service.ready is False


def test_serviceDisappearedWithoutServiceInfoDoesNotUpdateRegisteredServices(
    dns_sd, sd, zeroconf_without_service_info
):
    service = dns_sd.Service(type_='halrcomp')
    sd.register(service)
    service.ready = True

    sd.remove_service(
        zeroconf_without_service_info,
        '_machinekit._tcp.local.',
        'Foo on Bar 127.0.0.1._machinekit._tcp.local.',
    )

    assert service.ready is True


def test_serviceInfoSetsAllRelevantValuesOfService(dns_sd):
    service = dns_sd.Service(type_='halrcomp')
    service_info = ServiceInfoFactory().create(
        name='Foo on Bar',
        uuid=b'987654321',
        version=5,
        host='10.0.0.10',
        protocol='tcp',
        port=12456,
        server='sandybox.local',
    )

    service.add_service_info(service_info)

    assert service.uri == 'tcp://10.0.0.10:12456'
    assert service.name == service_info.name
    assert service.uuid == '987654321'
    assert service.version == 5
    assert service.host_name == 'sandybox.local'
    assert service.host_address == '10.0.0.10'


def test_serviceInfoResolvesLocalHostnameIfMatched(dns_sd):
    service = dns_sd.Service(type_='halrcomp')
    service_info = ServiceInfoFactory().create(
        host='sandybox.local',
        protocol='tcp',
        port=12456,
        server='sandybox.local',
        address='10.0.0.10',
    )

    service.add_service_info(service_info)

    assert service.uri == 'tcp://10.0.0.10:12456'


def test_serviceInfoRetursRawUriIfHostnameIsNotMatched(dns_sd):
    service = dns_sd.Service(type_='halrcomp')
    service_info = ServiceInfoFactory().create(
        host='thinkpad.local',
        protocol='tcp',
        port=12456,
        server='sandybox.local',
        address='10.0.0.10',
    )

    service.add_service_info(service_info)

    assert service.uri == 'tcp://thinkpad.local:12456'


def test_serviceInfoWithIncompleteValuesIsIgnoredByService(dns_sd):
    service = dns_sd.Service(type_='launcher')
    service_info = ServiceInfoFactory().create(properties={})

    service.add_service_info(service_info)

    assert service.uri == ''
    assert service.uuid == ''
    assert service.version == b''


def test_removingServiceInfoResetsAllRelevantValuesOfService(dns_sd):
    service = dns_sd.Service(type_='blahus')
    service_info = ServiceInfoFactory().create()
    service.add_service_info(service_info)

    service.remove_service_info(service_info)

    assert service.uri == ''
    assert service.name == ''
    assert service.uuid == ''
    assert service.version == 0
    assert service.host_name == ''
    assert service.host_address == ''


def test_clearingServiceInfosResetsValuesOfService(dns_sd):
    service = dns_sd.Service(type_='foobar')
    service.add_service_info(ServiceInfoFactory().create())
    service.add_service_info(ServiceInfoFactory().create())

    service.clear_service_infos()

    assert service.ready is False
    assert service.uri == ''


def test_settingReadyPropertyOfServiceTriggersCallback(dns_sd):
    cb_called = [False]

    def cb(_):
        cb_called[0] = True

    service = dns_sd.Service(type_='halrcomp')
    service.on_ready_changed.append(cb)
    service_info = ServiceInfoFactory().create()

    service.add_service_info(service_info)

    assert cb_called[0] is True


def test_discoverableAddingServiceWorks(dns_sd):
    discoverable = dns_sd.ServiceContainer()
    service = dns_sd.Service(type_='foo')

    discoverable.add_service(service)

    assert service in discoverable.services


def test_discoverableAddingAnythingElseFails(dns_sd):
    discoverable = dns_sd.ServiceContainer()
    item = object()

    try:
        discoverable.add_service(item)
        assert False
    except TypeError:
        assert True

    assert item not in discoverable.services


def test_discoverableRemovingServiceWorks(dns_sd):
    discoverable = dns_sd.ServiceContainer()
    service = dns_sd.Service(type_='foo')

    discoverable.add_service(service)
    discoverable.remove_service(service)

    assert service not in discoverable.services


def test_discoverableRemvoingAnythingElseFails(dns_sd):
    discoverable = dns_sd.ServiceContainer()
    item = object()

    try:
        discoverable.remove_service(item)
        assert False
    except TypeError:
        assert True

    assert item not in discoverable.services


def test_discoverableAllServicesReadySetServicesReady(dns_sd):
    discoverable = dns_sd.ServiceContainer()
    service1 = dns_sd.Service(type_='foo')
    discoverable.add_service(service1)
    service2 = dns_sd.Service(type_='bar')
    discoverable.add_service(service2)

    service1.ready = True
    service2.ready = True

    assert discoverable.services_ready is True


def test_discoverableNotAllServicesReadyUnsetsServicesReady(dns_sd):
    discoverable = dns_sd.ServiceContainer()
    service1 = dns_sd.Service(type_='foo')
    discoverable.add_service(service1)
    service2 = dns_sd.Service(type_='bar')
    discoverable.add_service(service2)

    service1.ready = True
    service2.ready = True
    service1.ready = False

    assert discoverable.services_ready is False


def test_discoverableServicesReadyChangedCallsCallback(dns_sd):
    cb_called = [False]

    def cb(_):
        cb_called[0] = True

    discoverable = dns_sd.ServiceContainer()
    discoverable.on_services_ready_changed.append(cb)

    discoverable.services_ready = True

    assert cb_called[0] is True


def test_serviceDiscoveryFilterAcceptCorrectUuid(dns_sd):
    service_info = ServiceInfoFactory().create(uuid=b'987654321')
    filter = dns_sd.ServiceDiscoveryFilter(txt_records={b'uuid': b'987654321'})

    assert filter.matches_service_info(service_info) is True


def test_serviceDiscoveryFilterRejectWrongUuid(dns_sd):
    service_info = ServiceInfoFactory().create(uuid=b'123456789')
    filter = dns_sd.ServiceDiscoveryFilter(txt_records={b'uuid': b'987654321'})

    assert filter.matches_service_info(service_info) is False


def test_serviceDiscoveryFilterAcceptFuzzyName(dns_sd):
    service_info = ServiceInfoFactory().create(name='Hello World')
    filter = dns_sd.ServiceDiscoveryFilter(name='Hello')

    assert filter.matches_service_info(service_info) is True


def test_serviceDiscoveryFilterAcceptExactMatchingName(dns_sd):
    service_info = ServiceInfoFactory().create(name='Foo')
    filter = dns_sd.ServiceDiscoveryFilter(name='Foo')

    assert filter.matches_service_info(service_info) is True


def test_serviceDiscoveryFilterRejectNonMatchingName(dns_sd):
    service_info = ServiceInfoFactory().create(name='Carolus Rex')
    filter = dns_sd.ServiceDiscoveryFilter(name='Adolfus Maximus')

    assert filter.matches_service_info(service_info) is False


def test_serviceDiscoveryFilterPassingWrongObjectFails(dns_sd):
    filter = dns_sd.ServiceDiscoveryFilter()

    try:
        filter.matches_service_info(object())
        assert False
    except TypeError:
        assert True


def test_serviceDiscoveryFiltersOutDiscoveredServiceWithWrongUuid(dns_sd, sd, zeroconf):
    service = dns_sd.Service(type_='halrcomp')
    sd.register(service)
    sd.filter = dns_sd.ServiceDiscoveryFilter(txt_records={b'uuid': b'87654321'})

    sd.add_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'Machinekit on MyBox 12.0.0.1._machinekit._tcp.local.',
    )

    assert service.ready is False


def test_serviceDiscoveryFiltersInDiscoveredServiceWithCorrectUuid(
    dns_sd, sd, zeroconf
):
    service = dns_sd.Service(type_='halrcomp')
    sd.register(service)
    sd.filter = dns_sd.ServiceDiscoveryFilter(txt_records={b'uuid': b'12345678'})

    sd.add_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'SuperPrint 192.168.7.2._machinekit._tcp.local.',
    )
    assert service.ready is True


def test_serviceDiscoveryFiltersInDisappearedServiceWithCorrectUuid(
    dns_sd, sd, zeroconf
):
    service = dns_sd.Service(type_='halrcomp')
    sd.register(service)
    sd.filter = dns_sd.ServiceDiscoveryFilter(txt_records={b'uuid': b'12345678'})

    sd.add_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'SuperPrint 192.168.7.2._machinekit._tcp.local.',
    )
    sd.remove_service(
        zeroconf,
        '_machinekit._tcp.local.',
        'SuperPrint 192.168.7.2._machinekit._tcp.local.',
    )
    assert service.ready is False



#!/usr/bin/env python
# coding=utf-8

import time
import sys
import threading

from pymachinetalk.dns_sd import ServiceDiscovery
import pymachinetalk.halremote as halremote


class BasicClass(object):
    def __init__(self):
        self.sd = ServiceDiscovery()

        rcomp = halremote.RemoteComponent('anddemo', debug=False)
        rcomp.no_create = True
        rcomp.newpin('button0', halremote.HAL_BIT, halremote.HAL_OUT)
        rcomp.newpin('button1', halremote.HAL_BIT, halremote.HAL_OUT)
        led_pin = rcomp.newpin('led', halremote.HAL_BIT, halremote.HAL_IN)
        led_pin.on_value_changed.append(self.led_pin_changed)
        led_pin.on_synced_changed.append(self.led_pin_synced)
        rcomp.on_connected_changed.append(self._connected)

        self.halrcomp = rcomp
        self.sd.register(rcomp)

    def led_pin_synced(self, synced):
        if synced:
            print("LED pin synced")

    def led_pin_changed(self, value):
        print('LED pin value changed: %s' % str(value))

    def _connected(self, connected):
        print('Remote component connected: %s' % str(connected))

    def start(self):
        self.sd.start()

    def stop(self):
        self.sd.stop()


def main():
    basic = BasicClass()

    print('starting')
    basic.start()

    try:
        while True:
            time.sleep(0.5)
    except KeyboardInterrupt:
        pass

    print('stopping threads')
    basic.stop()

    # wait for all threads to terminate
    while threading.active_count() > 1:
        time.sleep(0.1)

    print('threads stopped')
    sys.exit(0)


if __name__ == "__main__":
    main()
